/*Network monitor for system tray*/
/*(c) Copyright Barry Kauler 2010, bkhome.org*/
/*GPL license: /usr/share/doc/legal/gpl-2.0.txt*/
/*first release, version 2.0, may 18, 2010*/
/*version 2.1: new icons*/
/*version 2.3: icons now external, now displays dead icon correctly*/
/*version 2.3.2: still trying to get icon to update correctly*/
/*version 2.4: display tx/rx in KB if <1024, improved mouse-click operations*/
/*version 2.4.1: fix icon update when disconnect/reconnect*/
/*version 2.4.2: display acc monthly tx/rx*/
/*version 2.4.3: change 'long int' to 'long long int' -- 32-bit to 64-bit*/
/*2.5 (20120519): rodin.s: added gettext*/
/*2.6 (20130101) rerwin: added frisbee support, for multiple active network types*/
/*2.7 (20130405) rerwin: added multiple active network type icons*/
/*2.7.2 (20130708) 01micko: added IP address to tooltip; added icons to right-click menu. (no 2.7.1)*/
/*2.7.3 (20141104) rcrsn51: changed all 'atol' to 'atoll' to match 'long long' byte counts.*/
/*2.7.4 (20150401) rerwin: set undefined Update cyclic function return status to TRUE, to ensure cycling occurs; replace use of deprecated gtk_timeout_add with g_timeout_add and gtk_status_icon_set_tooltip with gtk_status_icon_set_tooltip_text.*/

#include <string.h>
#include <libintl.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gstdio.h>
#include <dirent.h>
#include <errno.h>
#define _(STRING)    gettext(STRING)

//2.7...
#define number_of_if_types 8
#define number_of_if_transfers 5
#define number_of_if_slots 10	
char *icondir="/usr/local/lib/X11/mini-icons/network_tray";
char *dialuptypefile="/tmp/.network_tray-use_analog_dialup_icons"; //from dialer (pupdial)
char *multinetworkfile="/tmp/.network_tray-use_wireless_control_menu_labels"; //from network manager (frisbee)
char *if_transfer_string[number_of_if_transfers]={"active", "none", "in", "out", "both"};
enum transfer_type {dormant=-1, active, none, in, out, both};
enum transfer_type if_transfer, itransfer;
enum if_status_type {not_detected=0, detected=1, not_dormant=2, bound=4, connected=7};
enum if_status_type best_if_type_status;
struct if_type_icon_data_element {
	char if_type_string[15];
	enum if_status_type if_type_status;
	GdkPixbuf *pixbuf[number_of_if_transfers];
};
struct if_type_icon_data_element if_type_icon_data[number_of_if_types]={{"pots"}, {"mobilewifieth"}, {"mobilewifi"}, {"mobileeth"}, {"wifieth"}, {"mobile"}, {"wifi"}, {"ethernet"}};
enum if_type_type {pots, mobilewifieth, mobilewifi, mobileeth, wifieth, mobile, wifi, ethernet};
enum if_type_type itype, best_if_type, best_if_type_prev=-1;
struct if_data_element {
	char if_string[15];
	enum if_status_type if_status;
	enum if_type_type if_type;
	int *if_type_element_ptr;
};
struct if_data_element if_data[number_of_if_slots];
int new_if_data_slot=0;
int iif;
DIR *dip2;
GdkPixbuf *dead_pixbuf;

GtkStatusIcon *tray_icon;
unsigned int interval = 500; /*update interval in milliseconds*/

char *netdir="/sys/class/net";
DIR *dip;
struct dirent *dit;
//int  i = 0;
char buf1[128]; //2.7
char buf2[128]; //2.7
FILE *fp;
char char1;
int flagactive=0;
int flagactiveprev=0;
char infomsg[256];
char rxstr[52];
long long int rxacc=0;
char rxaccstr[64];
char txstr[52];
long long int txacc=0;
char txaccstr[64];
long long int rxaccprev=0;
long long int txaccprev=0;
enum transfer_type flagtransfer=active, flagtransferprev=active; //2.7
int loopcnt=0;
int breakcnt=0;
int flagdisconnect=0;

//100814
char *rxmonthfile="/var/local/sns/rx_bytes_month";
char *txmonthfile="/var/local/sns/tx_bytes_month";
char rxstrmonth[52];
long long int rxaccmonth=0;
char txstrmonth[52];
long long int txaccmonth=0;
long long int rxaccmonth_updated=0;
long long int txaccmonth_updated=0;
char ipa[20];

GError *gerror;


gboolean Update(gpointer ptr);
int find_active();

int find_active() {

	//ip address
	fp = (FILE *)popen("/sbin/ifconfig|grep -iE 'Bcast|P-t-P'|tr -s ' '|tr ' ' ':'|cut -d ':' -f4" , "r");
	fgets(ipa,sizeof ipa,fp);
	pclose(fp);

	if (ipa==NULL) return 1;
    
    flagactive=0;
	rxacc=0; //2.7
	txacc=0; //2.7
    infomsg[0]=0;
    strcat(infomsg,_("Active interfaces: "));
	new_if_data_slot=0; //2.7
	for (itype=0; itype<number_of_if_types; itype++) if_type_icon_data[itype].if_type_status=not_detected; //2.7
    
    dip=opendir(netdir);
    if (dip==NULL) return 1;
    while ((dit = readdir(dip)) != NULL)
    {
        //i++;
        if (strcmp(dit->d_name,".")!=0 && strcmp(dit->d_name,"..")!=0 && strcmp(dit->d_name,"lo")!=0 ) {
			//2.7...
			strcpy (if_data[new_if_data_slot].if_string, dit->d_name);

            strcpy(buf1,netdir);
            strcat(buf1,"/");
            strcat(buf1,dit->d_name);
//2.7...
            strcpy(buf2,buf1);
            strcat(buf2,"/wireless");
			dip2=opendir(buf2);
			if (dip2!=NULL) {
				if_data[new_if_data_slot].if_type=wifi;
				closedir(dip2);
			}
			else if (strncmp(dit->d_name,"ppp",3)==0 && strncmp(dit->d_name,"pppoe",5)!=0) {
				fp=fopen(dialuptypefile,"r");
				if (fp!=NULL) {
					if_data[new_if_data_slot].if_type=pots;
					fclose(fp);
				}
				else if_data[new_if_data_slot].if_type=mobile;
			}
			else {
				strcpy(buf2,buf1);
				strcat(buf2,"/device");
				dip2=opendir(buf2);
				if (dip2!=NULL) {
					if_data[new_if_data_slot].if_type=ethernet;
					closedir(dip2);
				}
				else continue;
			}
			if_data[new_if_data_slot].if_status=detected;

            strcpy(buf2,buf1);
            strcat(buf2,"/dormant");
            fp=fopen(buf2,"r");
            
            if (fp!=NULL) {
                char1=fgetc(fp);
                fclose(fp);
                //if interface active, char1==0...
                if (char1=='0') {
//2.7...
					if_data[new_if_data_slot].if_status|=not_dormant;
					strcpy(buf2,buf1);
					strcat(buf2,"/carrier");
					fp=fopen(buf2,"r");
					if (fp!=NULL) {
						char1=fgetc(fp);
						fclose(fp);
						if (char1=='1') {
							if (if_data[new_if_data_slot].if_type == ethernet \
							 || if_data[new_if_data_slot].if_type == wifi) {
								strcpy(buf2,"/tmp/.network_tray-");
								strcat(buf2,dit->d_name);
								strcat(buf2,"_bound");
								fp=fopen(buf2,"r");
								if (fp!=NULL) {
									fclose(fp);
									if_data[new_if_data_slot].if_status|=bound;
								}
							}
							else if_data[new_if_data_slot].if_status|=bound;
						}
					}
//printf("if_data: %s, %i, %i\n",if_data[new_if_data_slot].if_string,if_data[new_if_data_slot].if_status,if_data[new_if_data_slot].if_type); fflush(stdout); //TEST
					
                    flagactive=flagactive+1;
                    strcat(infomsg,dit->d_name);
                    strcat(infomsg," ");
                    //accumulate bytes received...
                    strcpy(buf1,netdir); //2.7
                    strcat(buf1,"/");
                    strcat(buf1,dit->d_name);
                    strcat(buf1,"/statistics/rx_bytes");
                    fp=fopen(buf1,"r");
                    if (fp!=NULL) {
                        fgets(rxstr,12,fp);
                        rxacc+=atoll(rxstr)/1024; //2.7
                        fclose(fp);
                    }
                    rxaccmonth_updated=(rxaccmonth+rxacc)/1024; //100814 in MB
                    //accumulate bytes transmitted...
                    buf1[0]=0;
                    strcat(buf1,netdir);
                    strcat(buf1,"/");
                    strcat(buf1,dit->d_name);
                    strcat(buf1,"/statistics/tx_bytes");
                    fp=fopen(buf1,"r");
                    if (fp!=NULL) {
                        fgets(txstr,12,fp);
                        txacc+=atoll(txstr)/1024; //2.7
                        fclose(fp);
                    }
                    txaccmonth_updated=(txaccmonth+txacc)/1024; //100814 in MB
                }
            }
			new_if_data_slot++; //2.7
        }
    }
//2.7...
    best_if_type_status=not_detected;
    best_if_type=-1;
	for (iif=0; iif<new_if_data_slot; iif++) {
		if (if_data[iif].if_type<number_of_if_types) {
			if_type_icon_data[if_data[iif].if_type].if_type_status|=if_data[iif].if_status;
			best_if_type_status|=if_data[iif].if_status;
		}
	}
	if (if_type_icon_data[mobile].if_type_status==if_type_icon_data[wifi].if_type_status)
		if_type_icon_data[mobilewifi].if_type_status=if_type_icon_data[mobile].if_type_status;
	if (if_type_icon_data[mobile].if_type_status==if_type_icon_data[ethernet].if_type_status)
		if_type_icon_data[mobileeth].if_type_status=if_type_icon_data[mobile].if_type_status;
	if (if_type_icon_data[wifi].if_type_status==if_type_icon_data[ethernet].if_type_status)
		if_type_icon_data[wifieth].if_type_status=if_type_icon_data[wifi].if_type_status;
	if (if_type_icon_data[mobilewifi].if_type_status==if_type_icon_data[mobileeth].if_type_status)
		if_type_icon_data[mobilewifieth].if_type_status=if_type_icon_data[mobilewifi].if_type_status;
	for (itype=0; itype<number_of_if_types; itype++) {
		if (if_type_icon_data[itype].if_type_status==best_if_type_status) {
			best_if_type=itype;
			break;
		}
	}

//printf("if_type_status: %i, %i, %i, %i, %i, %i, %i, %i  best_if_type_status: %i  best_if_type: %i\n",if_type_icon_data[0].if_type_status,if_type_icon_data[1].if_type_status,if_type_icon_data[2].if_type_status,if_type_icon_data[3].if_type_status,if_type_icon_data[4].if_type_status,if_type_icon_data[5].if_type_status,if_type_icon_data[6].if_type_status,best_if_type_status,if_type_icon_data[7].if_type_status,best_if_type_status,best_if_type); fflush(stdout); //TEST

    closedir(dip);
    if (flagactive==0) {
        strcat(infomsg,_("none "));
        strcat(infomsg,_(" Click this icon for network setup"));
    }
    else {
        if (rxacc < 1024) {
            sprintf(rxaccstr,"%.1f",((float)rxacc));
            strcat(infomsg,"\nRx: "); //2.7
            strcat(infomsg,rxaccstr);
            strcat(infomsg,_("KB "));
        }
        else {
            sprintf(rxaccstr,"%.1f",((float)rxacc/1024.0));
            strcat(infomsg,"\nRx: "); //2.7
            strcat(infomsg,rxaccstr);
            strcat(infomsg,_("MB "));
        }
        //100814 monthly acc...
//        sprintf(rxaccstr,_("(Month:%dMB) "),rxaccmonth_updated);
        sprintf(rxaccstr,_("(Month: %lluMB) "),rxaccmonth_updated); //2.7
        strcat(infomsg,rxaccstr);
        if (txacc < 1024) {
            sprintf(txaccstr,"%.1f",((float)txacc));
            strcat(infomsg,"\nTx: "); //2.7
            strcat(infomsg,txaccstr);
            strcat(infomsg,"KB ");
        }
        else {
            sprintf(txaccstr,"%.1f",((float)txacc/1024.0));
            strcat(infomsg,"\nTx: "); //2.7
            strcat(infomsg,txaccstr);
            strcat(infomsg,_("MB "));
        }
        //100814 monthly acc...
//        sprintf(txaccstr,_("(Month:%dMB) "),txaccmonth_updated);
        sprintf(txaccstr,_("(Month: %lluMB) "),txaccmonth_updated); //2.7
        strcat(infomsg,txaccstr);
        
        if (ipa!=NULL) {
			strcat(infomsg,_("\nIP address: "));
			strcat(infomsg,ipa);
		}
    }
}

gboolean Update(gpointer ptr) {

    find_active();
    loopcnt=loopcnt+1;
    
    if (flagactive==0) flagtransfer=dormant; //2.7...
    else {
		if (best_if_type_status >= bound) {

			flagtransfer=none;
			if (rxacc!=rxaccprev) flagtransfer=in;
			if (txacc!=txaccprev) flagtransfer=out;
			if (txacc!=txaccprev && rxacc!=rxaccprev) flagtransfer=both;
		}
        else  flagtransfer=active; //2.7
    }
//printf("loopcnt=%i flagtransfer=%i flagactive=%i\n",loopcnt,flagtransfer,flagactive); fflush(stdout); //TEST
    //lot of trouble with this logic, so introduce breakcnt to force icon update if stuck...
    if (flagtransferprev == flagtransfer) {
        if (loopcnt != 1) { //want to update icon on first loop.
            if (flagactive == flagactiveprev && flagtransfer == flagtransferprev && flagtransfer == none) breakcnt=breakcnt+1; //no change.
            if (flagactive == flagactiveprev && flagtransfer == flagtransferprev && flagactive == 0) breakcnt=breakcnt+1; //no change.
        }
        else breakcnt=0;
        if (breakcnt != 0 && breakcnt < 8) return; //force update after 4 seconds.
    }
    breakcnt=0;
    flagactiveprev=flagactive;
    txaccprev=txacc;
    rxaccprev=rxacc;
    flagtransferprev=flagtransfer;
    best_if_type_prev=best_if_type; //2.7
	if (flagtransfer == dormant) gtk_status_icon_set_from_pixbuf(tray_icon,dead_pixbuf); //2.7
	else gtk_status_icon_set_from_pixbuf(tray_icon,if_type_icon_data[best_if_type].pixbuf[flagtransfer]); //2.7
//printf("gtk_status_icon_set_from_pixbuf: best_type: %i  best_status: %i  flagtransfer: %i\n",best_if_type,best_if_type_status,flagtransfer); fflush(stdout); //TEST
   
    //update tooltip...
#if GTK_CHECK_VERSION(2, 15, 3) //2.7.4...
    gtk_status_icon_set_tooltip_text(tray_icon, infomsg);
#else
    gtk_status_icon_set_tooltip(tray_icon, infomsg);
#endif
return TRUE; //2.7.4 end
}

void  view_popup_menu_onSetupNetworking (GtkWidget *menuitem, gpointer userdata);
void  view_popup_menu_onSetupNetworking (GtkWidget *menuitem, gpointer userdata)
  {
    /* we passed the view as userdata when we connected the signal */
    /*GtkTreeView *treeview = GTK_TREE_VIEW(userdata);*/
    system("defaultconnect & ");
  }

void  view_popup_menu_onNetworkStatus (GtkWidget *menuitem, gpointer userdata);
void  view_popup_menu_onNetworkStatus (GtkWidget *menuitem, gpointer userdata)
  {
    system("ipinfo & ");
  }

void  view_popup_menu_onDisconnect (GtkWidget *menuitem, gpointer userdata);
void  view_popup_menu_onDisconnect (GtkWidget *menuitem, gpointer userdata)
  {
    flagdisconnect=1;
    system("/usr/local/apps/Connect/AppRun --disconnect & ");
  }

void  view_popup_menu_onReconnect (GtkWidget *menuitem, gpointer userdata);
void  view_popup_menu_onReconnect (GtkWidget *menuitem, gpointer userdata)
{
    system("/usr/local/apps/Connect/AppRun --connect & ");
    flagactiveprev=0; /*100703*/
}

void tray_icon_on_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data)
{
        GtkWidget *menu, *menuitem, *icon;
        menu = gtk_menu_new();
        menuitem = gtk_image_menu_item_new_with_label(_("Setup networking"));
        icon = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), icon);
		g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onSetupNetworking, status_icon);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
        menuitem = gtk_image_menu_item_new_with_label(_("Network status information"));
        icon = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_MENU);
		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), icon);
		g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onNetworkStatus, status_icon);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
        //2.7...
		fp = fopen(multinetworkfile,"r");
		if (fp != NULL) {
			fclose(fp);
			if (if_type_icon_data[wifi].if_type_status == connected)
			{
				menuitem = gtk_image_menu_item_new_with_label(_("Disable wireless network"));
				icon = gtk_image_new_from_stock(GTK_STOCK_CANCEL, GTK_ICON_SIZE_MENU);
				gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), icon);
			}
			else 
			{
			menuitem = gtk_image_menu_item_new_with_label(_("Enable wireless network"));
			icon = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_MENU);
			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), icon);
			g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onDisconnect, status_icon);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
			}
		}
		else {
		
            if (flagactive != 0) {
                flagdisconnect=0;
                menuitem = gtk_image_menu_item_new_with_label(_("Disconnect from network"));
                icon = gtk_image_new_from_stock(GTK_STOCK_DISCONNECT, GTK_ICON_SIZE_MENU);
				gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), icon);
				g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onDisconnect, status_icon);
                gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
            }
            if (flagdisconnect == 1) {
                menuitem = gtk_image_menu_item_new_with_label(_("Reconnect to network"));
                icon = gtk_image_new_from_stock(GTK_STOCK_CONNECT, GTK_ICON_SIZE_MENU);
				gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), icon);
				g_signal_connect(menuitem, "activate", (GCallback) view_popup_menu_onReconnect, status_icon);
                gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
            }
       }
        gtk_widget_show_all(menu);
        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, gdk_event_get_time(NULL));
}

void tray_icon_on_click(GtkStatusIcon *status_icon, gpointer user_data)
{
    if (flagactive == 0)system("defaultconnect & ");
    else {
        system(_("yaf-splash -placement bottom -bg purple -timeout 10 -text 'Note: right-click icon for network menu' & "));
        system("ipinfo & ");
    }
}

static GtkStatusIcon *create_tray_icon() {
	char icontag[15]=""; //2.7

    tray_icon = gtk_status_icon_new();
    g_signal_connect(G_OBJECT(tray_icon), "activate", G_CALLBACK(tray_icon_on_click), NULL);
    g_signal_connect(G_OBJECT(tray_icon), "popup-menu", G_CALLBACK(tray_icon_on_menu), NULL);

//2.7...
	dip=opendir(icondir);
	if (dip==NULL) return NULL;
	while ((dit = readdir(dip)) != NULL) {
		if (strrchr(dit->d_name,'.') != NULL && strcmp(strrchr(dit->d_name,'.'),".") != 0) {
			strcpy(icontag,strrchr(dit->d_name,'.'));
			break;
		}
	}
    closedir(dip);
	for (itype=0; itype<number_of_if_types; itype++) {
		for (itransfer=0; itransfer<number_of_if_transfers; itransfer++) {
			strcpy(buf1,icondir);
			strcat(buf1,"/");
			strcat(buf1,if_type_icon_data[itype].if_type_string);
			strcat(buf1,"-");
			strcat(buf1,if_transfer_string[itransfer]);
			strcat(buf1,icontag);
			if_type_icon_data[itype].pixbuf[itransfer]= gdk_pixbuf_new_from_file(buf1,&gerror);
// printf("itype=%i itransfer=%i Icon_file_name=%s\n",itype,itransfer,buf1); fflush(stdout); //TEST
		}
	}
	strcpy(buf1,icondir);
	strcat(buf1,"/dead");
	strcat(buf1,icontag);
    dead_pixbuf=gdk_pixbuf_new_from_file(buf1,&gerror);
	gtk_status_icon_set_from_pixbuf(tray_icon,if_type_icon_data[wifieth].pixbuf[active]);

#if GTK_CHECK_VERSION(2, 15, 3) //2.7.4...
    gtk_status_icon_set_tooltip_text(tray_icon,_("No active network interfaces"));
#else
    gtk_status_icon_set_tooltip(tray_icon,_("No active network interfaces"));
#endif //2.7.4 end
    gtk_status_icon_set_visible(tray_icon, TRUE);
    return tray_icon;
}

int main(int argc, char **argv) {

    gtk_init(&argc, &argv);
    
    setlocale( LC_ALL, "" );
    bindtextdomain( "network_tray", "/usr/share/locale" );
    textdomain( "network_tray" );

    //100814 monthly acc (see also /usr/local/simple_network_setup/rc.network and rc.shutdown)...
    fp=fopen(rxmonthfile,"r");
    if (fp!=NULL) {
       fgets(rxstrmonth,12,fp);
       rxaccmonth=atoll(rxstrmonth)/1024; //in KB.
       fclose(fp);
    }
    fp=fopen(txmonthfile,"r");
    if (fp!=NULL) {
       fgets(txstrmonth,12,fp);
       txaccmonth=atoll(txstrmonth)/1024;
       fclose(fp);
    }

    tray_icon = create_tray_icon();
        
    g_timeout_add(interval, Update, NULL); //2.7.4

    gtk_main();
    return 0;
}
